#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ncurses.h>

#include "include/main.h"
#include "include/card.h"

void die(char *s) {
    fprintf(stderr, "fatal: %s\n", s);
    exit(1);
}
char *fgetl(FILE *stream) { size_t s_sz = 1; // +1 for null byte
    char *s = NULL;
    char c;

    while ((c = fgetc(stream)) != '\n') {
        if (c == EOF)
            return NULL;

        s = (char *)realloc((void *)s, ++s_sz);
        s[s_sz - 2] = c;
    }

    if (s != NULL)
        s[s_sz - 1] = '\0';

    return s;
}

char *strdup_(char *s) {
    char *r;
    r = (char *)malloc(strlen(s) + 1);
    strcpy(r, s);
    return r;
}

int main(int argc, char **argv) {
    CARD *cards;
    WINDOW *main_card, *outer_text_card, *inner_text_container;
    int cards_ct, cc = 0, cf = 0, max_y, max_x;
    char k = '\0', *help_message;

    if (argc < 2)
        die("filename required");

    cards = card_parse_csv(argv[1], &cards_ct);

    // curses init
    initscr();
    noecho();
    curs_set(0);
    if (!has_colors()) {
        endwin();
        die("terminal with color support required");
    }
    start_color();
    init_pair(1, COLOR_WHITE, COLOR_BLUE);
    init_pair(2, COLOR_BLACK, COLOR_WHITE);
    
    // main loop
    do {
        // (re)init cards
        getmaxyx(stdscr, max_y, max_x);
        main_card = newwin(max_y * .75,
                max_x * .75,
                (max_y * .5) - ((max_y * .75) * .5),
                (max_x * .5) - ((max_x * .75) * .5));

        getmaxyx(main_card, max_y, max_x);
        outer_text_card = derwin(main_card,
                max_y * .5,
                max_x * .9,
                (max_y * .5) - ((max_y * .5) * .5),
                (max_x * .5) - ((max_x * .9) * .5));

        getmaxyx(outer_text_card, max_y, max_x);
        inner_text_container = derwin(outer_text_card,
                max_y - 2,
                max_x - 2,
                1,
                1);

        // (re)draw cards
        clear();
        /* wattron(outer_text_card, A_BOLD); */
        wattron(inner_text_container, COLOR_PAIR(2));
        wattron(main_card, A_BOLD);
        box(outer_text_card, 0, 0);
        box(main_card, 0, 0);
        refresh();

        // get user input
        switch (k) {
            case 'j':
                if (++cc > cards_ct - 1)
                    cc = 0;
                cf = 0;
            break;
            case 'k':
                if (--cc < 0)
                    cc = cards_ct - 1;
                cf = 0;
            break;
            case ' ':
                switch (cf) {
                    case 0: cf = 1; break;
                    case 1: cf = 0; break;
                }
            break;
            case 'q':
                delwin(inner_text_container);
                delwin(outer_text_card);
                delwin(main_card);
                endwin();
                exit(0);
            break;
        }

        // print card # and text
        if (cf == 0) { // obverse
            mvwprintw(main_card, 0, 10, "  Card #%d/%d [front]  \r", cc + 1, cards_ct);
            wprintw(inner_text_container, "%s\r", cards[cc].obverse);
        } else { // reverse
            mvwprintw(main_card, 0, 10, "  Card #%d/%d [back]  \r", cc + 1, cards_ct);
            wprintw(inner_text_container, "%s\r", cards[cc].reverse);
        }

        // print help info
        help_message = "<j> next card   <k> prev card   <spc> flip card   <q> quit";
        getmaxyx(main_card, max_y, max_x);
        mvwprintw(main_card, max_y - 2, max_x - 2 - strlen(help_message), "%s", help_message);

        /// colors
        wbkgd(outer_text_card, COLOR_PAIR(2));
        wbkgd(main_card, COLOR_PAIR(1));

        // refresh
        touchwin(outer_text_card);
        wrefresh(inner_text_container);
        touchwin(main_card);
        wrefresh(outer_text_card);
        wrefresh(main_card);
        delwin(inner_text_container);
        delwin(outer_text_card);
        delwin(main_card);

    } while ((k = getch()) != EOF);

    return 0;
}
